class VizConnection extends TcpLink
	config(BotAPI);

var string ReceivedData;
var string RecievedArgs[9];
var string RecievedVals[9];

//used to store info inside function where local wont work
//set property text dosent seem to work with a local
var actor tempActor;
var VizServer Parent;

// set true to allow viz clients to pause
var config bool bAllowPause;

// set true for verbose debug logs
var config bool bDebug;

// delay between visionUpdates
var config float updateTime;

// record once we have sent navs - don't need to duplicate
var bool sentNavs;



// configurable delimeters for strings sent to clients
// currently of form (with multiple possible attributes):
// "type"$ib$"identifier"$ib$as$"attrname1"$ib$"attrvalue1"$ae

// as = identifies the start of an attribute
var string as;
// ae = attribute end
var string ae;
// ib = item break
var string ib;


//------------Message Types---------------------------
// Each message appears on a seperate line. The first item
//  of each message is the type identifier. It is a three
//  letter code identifing what the message contains.
// Used types are:
// (Asynchronous messages)
// (Synchronous messages)
//  PLR - see another player
//  INV - see an inventory item (or weapon)
//  NAV - see a navigation point
//  DOM - see a domination point
//  FLG - see a flag
//  END - end of a vision batch
//  BEG - begin of a vision batch
//  GAM - random information about the game
//--------------------------------------------------


function PostBeginPlay()
{
    Parent = VizServer(Owner);
    if(bDebug)
    	log("Spawned VizConnection");
}

//Socket established
event Accepted()
{
	local string gameInfoStr, levelName;
	local int i;
	
    if(bDebug)
    	log("Accepted VizConnection");
    	
 	//no clean way to do this since gametypes are not derived from
 	//a common ancestor
	switch(Parent.gameClass)
	{
		/*case "BotTeamGamePlus":
			gameInfoStr = BotTeamGamePlus(Level.Game).GetGameInfo();
			break;*/
		case "BotDeathMatch":
			gameInfoStr = BotDeathMatch(Level.Game).GetGameInfo();
			break;	
		/*case "BotDomination":
			gameInfoStr = BotDomination(Level.Game).GetGameInfo();
			break;	
		case "BotCTF":
			gameInfoStr = BotCTF(Level.Game).GetGameInfo();
			break;	*/
	}
	
	levelName = string(Level);
	i = InStr(Caps(levelName), ".LEVELINFO");
	if(i != -1)
	levelName = Left(levelName, i);

	SendLine("NFO" $ib$as$ "Gametype" $ib$ Parent.gameClass $ae$ib$as$ 
		"Level" $ib$ levelName $ae$ gameInfoStr);

    gotoState('monitoring','Running');
}

//Closed on other end
event Closed()
{
	Destroy();
}

//Recieve info - parse into lines and call RecievedLine
event ReceivedText( string Text )
{
	local int i;
	local string S;

    if(bDebug)
    	log("Recieved - "$Text);

	ReceivedData = ReceivedData $ Text;

	// remove a LF which arrived in a new packet
	// and thus didn't get cleaned up by the code below
	if(Left(ReceivedData, 1) == Chr(10))
		ReceivedData = Mid(ReceivedData, 1);
	i = InStr(ReceivedData, Chr(13));
	while(i != -1)
	{
		S = Left(ReceivedData, i);
		i++;
		// check for any LF following the CR.
		if(Mid(ReceivedData, i, 1) == Chr(10))
			i++;

		ReceivedData = Mid(ReceivedData, i);

		ReceivedLine(S);

		if(LinkState != STATE_Connected)
			return;

		i = InStr(ReceivedData, Chr(13));
	}
}





//Here is where we handle incoming commands
/* Commands expected to look like:
runto {Argument value} {Arg value}...
Currently hard coded to take no more than 9 args
Command type and arguments can be
any length, but first space terminates the name. Values can
have spaces or any other kind of character.
*/
function ReceivedLine(string S)
{    
	local string cmdType, argBody, rem;
	local int endloc, wordsplit, attrNum;
    if(bDebug)
    	log(S);

	wordsplit = InStr(S,ib);
	if( wordsplit == -1)
		wordsplit = Len(S);
		
	cmdType = left(S,wordsplit);	
	rem = mid(S,InStr(S,as));
	attrNum = 0;
	
	//iterate through attr/val pairs, storring them in the
	//parallel arrays RecievedArgs and RecievedVals
	while(attrNum < 9 && rem != "")
	{
		endloc = InStr(rem,ae);
		argBody = mid(rem,1,(endloc - 1));
		
		wordsplit = InStr(argBody,ib);
		RecievedArgs[attrNum] = left(argBody,wordsplit);
		RecievedVals[attrNum] = mid(argBody,(wordsplit + 1));
			
		rem = mid(rem,1); //advance
		rem = mid(rem,InStr(rem,as));
		attrNum++;
	}
	
   	cmdType = Caps(cmdType);
	
	ProcessAction(cmdType);
}


function string GetArgVal(string argName)
{
	local int i;
	while (i < 9 && RecievedArgs[i] != "")
	{
		if (RecievedArgs[i] ~= argName) 
			return RecievedVals[i];
		i++;
	}
	
	return "";
}


function ProcessAction(string cmdType)
{
	if(bDebug)
			log("comandType:"@cmdType);
	
	switch(cmdType)
	{
		case "PAUSE":
			if (bAllowPause)
				Level.Pauser.IsA('vizClient');
			break;
   		case "UNPAUSE":
			if (bAllowPause)
				Level.Pauser=None;
			break;
	}
}




//Send a line to the client
function SendLine(string Text, optional bool bNoCRLF)
{
    if(bDebug)
    	log("    Sending: "$Text);
	if(bNoCRLF)
		SendText(Text);
	else
		SendText(Text$Chr(13)$Chr(10));
}


//Concat two strings togeather with our list bookends
function string MakeItem(string first, string second)
{
	return (as$first$ib$second$ae);	
}



// iterate through objects and send appropriate messages
function checkGame()
{
	local Controller P;
local string plrString;
local NavigationPoint N;
	SendLine("BEG" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
 	//no clean way to do this since gametypes are not derived from
 	//a common ancestor that has AddRemoteBot
	switch(Parent.gameClass)
	{
		/*case "BotTeamGamePlus":
			sendLine( BotTeamGamePlus(Level.Game).GetGameStatus() );
			break;*/
		case "BotDeathMatch":
			sendLine( BotDeathMatch(Level.Game).GetGameStatus() );
			break;	
		/*case "BotDomination":
			sendLine( BotDomination(Level.Game).GetGameStatus() );
			break;	
		case "BotCTF":
			sendLine( BotCTF(Level.Game).GetGameStatus(none) );
			break;	*/
	}	
	
  for ( P=Level.ControllerList; P!=None; P=P.NextController)
{
  if(!P.PlayerReplicationInfo.bIsSpectator) {
    plrString = "PLR" $ib$as$ "Id" $ib$ P.PlayerReplicationInfo.PlayerName $ae$ib$as;
      if(P.IsA('RemoteBot')) {
	plrString = plrString $ "Rotation" $ib$ RemoteBot(P).myRotation $ae$ib$as;
      } else {
	plrString = plrString $ "Rotation" $ib$ P.Rotation $ae$ib$as;
      }
      //	      "ViewRotation" $ib$ PlayerController(P).ViewRotation $ae$ib$as$
      plrString = plrString $ 
      "Location" $ib$ P.Pawn.Location $ae$ib$as$
      "Team" $ib$ 0 $ae$ib$as$
      //				"Team" $ib$ P.PlayerReplicationInfo.Team $ae$ib$as$ 
	      "Name" $ib$ P.PlayerReplicationInfo.PlayerName $ae$ib$as$
      "Weapon" $ib$ P.Pawn.Weapon $ae;
			
    if( P.bFire != 0 )
      plrString = plrString $ib$as$ "Firing" $ib$ "1" $ae;
    else if( P.bAltFire != 0 )
      plrString = plrString $ib$as$ "AltFiring" $ib$ "2" $ae;
    else 
      plrString = plrString $ib$as$ "AltFiring" $ib$ "0" $ae;
	    sendLine(plrString);
  }
}

for ( N=Level.NavigationPointList; N!=None; N=N.NextNavigationPoint )
	{
		
		/*if(N.IsA('ControlPoint'))
		{
			if( ControlPoint(N).ControllingTeam == none )
				temp = 255;
			else
				temp = ControlPoint(N).ControllingTeam.TeamIndex;
			
			SendLine("DOM" $ib$as$ "Id" $ib$ N $ae$ib$as$ 
				"Location" $ib$ N.Location $ae$ib$as$ 
				"Controller" $ib$ temp $ae);
		}
		else if(!sentNavs)
		{*/
                     if(!sentNavs) 
		//only send Navs once
			SendLine("NAV" $ib$as$ "Id" $ib$ N $ae$ib$as$ 
				"Location" $ib$ N.Location $ae);
		//}
	}
		
	sentNavs = true;

	  /*	foreach AllActors (class'Inventory', I)
	{
		if( I.GetStateName() == 'Pickup' && !I.bHidden )
		{
			SendLine("INV" $ib$as$ "Id" $ib$ I $ae$ib$as$ 
				"Location" $ib$ I.Location $ae);
		}
	}

	foreach AllActors (class'CTFFlag', C)
	{
		outstring = "FLG" $ib$as$ "Id" $ib$ C $ae$ib$as$ 
				"Team" $ib$ C.Team $ae$ib$as$
				"State" $ib$ C.GetStateName() $ae;
				
		//when a flag is held its location is not updated by engine =(
		if( C.GetStateName() == 'Held') 
			outstring = outstring $ib$as$ "Location" $ib$ C.Holder.Location $ae$ib$as$ 
					"Holder" $ib$ C.Holder $ae;
		else
			outstring = outstring $ib$as$ "Location" $ib$ C.Location $ae;
			
		SendLine(outstring);
		}
*/

	SendLine("END" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
}




//fire right up into the loop for sending updates
auto state monitoring
{
Begin:
Running:
	if( Level.Pauser == None )
		checkGame();
    sleep(updateTime);
    goto 'Running';
}

defaultproperties
{
    bAllowPause=True
    updateTime=0.200000
    as="{"
    ae="}"
    ib=" "
}
