/* USARSim P2AT robot.
 * Stereo Vision added by:
 * Marco Zaratti - marco.zaratti@gmail.com
*/
class P2AT extends KRobot config(USAR);

var float cachedLeftValue,cachedRightValue;
var rotator cachedCameraRot,initCameraRot;
var bool bGetInitCameraRot;
var config bool bAbsoluteCamera;

///////////////////////////
// Added for stereo vision
///////////////////////////
var config vector stereoSpacing; //(eye to eye)/2 spacing in UU
///////////////////////////

function ProcessCarInput()
{
	local float LeftValue,RightValue;
	local int CameraPan,CameraTilt;

	Super.ProcessCarInput();

	if (!bGetInitCameraRot) {
		if (myCamera!=None)
			initCameraRot = myCamera.Rotation;
		USARRemoteBot(Controller).LeftThrottle = 0.0;
		USARRemoteBot(Controller).RightThrottle = 0.0;
		USARRemoteBot(Controller).myCameraRotation = rot(0,0,0);
		bGetInitCameraRot = true;
	}

	if (USARRemoteBot(Controller).Normalized) {
		LeftValue = USARRemoteBot(Controller).LeftThrottle*690;
		RightValue = USARRemoteBot(Controller).RightThrottle*690;
	} else {
		LeftValue = Converter.SpinSpeedToUU(USARRemoteBot(Controller).LeftThrottle);
		RightValue = Converter.SpinSpeedToUU(USARRemoteBot(Controller).RightThrottle);
	}

	if (cachedLeftValue!=LeftValue) {
		JointsControl[1].state = 1; // new command
		JointsControl[1].order = 1;
		JointsControl[1].value = LeftValue;
		JointsControl[3].state = 1; // new command
		JointsControl[3].order = 1;
		JointsControl[3].value = LeftValue;
		bNewCommand = true;
		cachedLeftValue=LeftValue;
	}
	if (cachedRightValue!=RightValue) {
		JointsControl[0].state = 1; // new command
		JointsControl[0].order = 1;
		JointsControl[0].value = RightValue;
		JointsControl[2].state = 1; // new command
		JointsControl[2].order = 1;
		JointsControl[2].value = RightValue;
		bNewCommand = true;
		cachedRightValue=RightValue;
	}
	if (myCamera!=None) {
		if (USARRemoteBot(Controller).myCameraRotation.roll!=0) return;
		USARRemoteBot(Controller).myCameraRotation.roll = -1;
		if (USARRemoteBot(Controller).myCameraRotationOrder==1) {
			CameraTilt = USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw;
		} else if (!bAbsoluteCamera) {
			CameraTilt = -USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw;
		} else {
			CameraTilt = myCamera.Rotation.Pitch - initCameraRot.Pitch -Rotation.Pitch - USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw - myCamera.Rotation.Yaw + initCameraRot.Yaw + Rotation.Yaw;
			if (CameraTilt>65535) CameraTilt-=65535;
			else if (CameraTilt<-65535) CameraTilt+=65535;
			if (CameraTilt>32768) CameraTilt-=65535;
			else if (CameraTilt<-32768) CameraTilt+=65535;
			if (CameraPan>65535) CameraPan-=65535;
			else if (CameraPan<-65535) CameraPan+=65535;
			if (CameraPan>32768) CameraPan-=65535;
			else if (CameraPan<-32768) CameraPan+=65535;
		}
		if (CameraPan!=0 || USARRemoteBot(Controller).myCameraRotationOrder==1) {
			JointsControl[5].state = 1;
			//JointsControl[5].steer = 65535;
			JointsControl[5].order = USARRemoteBot(Controller).myCameraRotationOrder;
			JointsControl[5].value = CameraPan;
		}
		if (JointsControl[5].order==1)
			PanSpeed = CameraPan;
		else
			PanSpeed = MotorSpeed;
		if (CameraTilt!=0 || USARRemoteBot(Controller).myCameraRotationOrder==1) {
			JointsControl[6].state = 1;
			//JointsControl[6].steer = 65535;
			JointsControl[6].order = USARRemoteBot(Controller).myCameraRotationOrder;
			JointsControl[6].value = CameraTilt;
		}
		if (JointsControl[6].order==1)
			TiltSpeed = CameraTilt;
		else
			TiltSpeed = MotorSpeed;
		bNewCommand = true;
	}
	//bReady = true;
}

///////////////////////////
// Added for stereo vision
///////////////////////////
simulated function DrawHud(Canvas C)
{
	local int halfSizeX;
	local vector stereoRotated;

	halfSizeX = C.SizeX/2.0;
	stereoRotated =  stereoSpacing >> myCamera.Rotation;

	C.Reset();
	C.Clear();	//Clears frame & Z buffers

	//left Eye:
	C.DrawPortal(0, 0, halfSizeX, C.SizeY, myCamera, myCamera.Location - stereoRotated, myCamera.Rotation, CameraZoom);

	//Right Eye:
	C.DrawPortal(halfSizeX, 0, halfSizeX, C.SizeY, myCamera, myCamera.Location + stereoRotated, myCamera.Rotation, CameraZoom);

	super.DrawHud(C);
}
///////////////////////////

defaultproperties
{
	///////////////////////////
	// Added for stereo vision
	///////////////////////////
	stereoSpacing=(Y=12.5) //5 cm: (eye to eye)/2 spacing in UU
	///////////////////////////

	bDebug=false
	StaticMesh=StaticMesh'USAR_Meshes.Robot.P2AT'
	DrawScale=1.0
	DrawScale3D=(X=1.0,Y=1.0,Z=1.0)


	bCollideActors=True
	CollisionRadius=64
	CollisionHeight=64

	ChassisMass=1.000000

	TireRollFriction=15.000000
	TireLateralFriction=15.000000
	TireRollSlip=0.0600
	TireLateralSlip=0.0600000
	TireMinSlip=0.001000
	TireSlipRate=0.00050000
	TireSoftness=0.000020
	TireAdhesion=0.000000
	TireRestitution=0.000000

    Begin Object Class=KarmaParamsRBFull Name=KParams0
        KInertiaTensor(0)=0.010000
        KInertiaTensor(3)=0.02000
        KInertiaTensor(5)=0.03000
        KCOMOffset=(X=0.0,Z=-0.3)
        KLinearDamping=0.000000
        KAngularDamping=0.000000
        KStartEnabled=True
        bHighDetailOnly=False
        bClientOnly=False
        bKDoubleTickRate=True
        KFriction=1.600000
        Name="KParams0"
    End Object
    KParams=KarmaParamsRBFull'USARBot.P2AT.KParams0'

	JointParts(0)=(PartName="LeftFWheel",PartClass=class'USARBot.P2ATTire',bSteeringLocked=True,bSuspensionLocked=true,Parent="",JointClass=class'KCarWheelJoint',ParentPos=(X=4.2,Y=7.350,Z=-2.1),ParentAxis=(Z=1.0),SelfPos=(Z=0.0),SelfAxis=(Z=1.0))
	JointParts(1)=(PartName="RightFWheel",PartClass=class'USARBot.P2ATTire',bSteeringLocked=True,bSuspensionLocked=true,Parent="",JointClass=class'KCarWheelJoint',ParentPos=(X=4.2,Y=-7.350,Z=-2.1),ParentAxis=(Z=1.0),SelfPos=(Z=0.0),SelfAxis=(Z=1.0))
	JointParts(2)=(PartName="RearWheel",PartClass=class'USARBot.P2ATTire',bSteeringLocked=True,bSuspensionLocked=true,Parent="",JointClass=class'KCarWheelJoint',ParentPos=(X=-7.35,Y=0.0,Z=-3.150),ParentAxis=(Z=1.0),SelfPos=(Z=0.0),SelfAxis=(Z=1.0))
}
