class P2DX extends KRobot config(USAR);

var float cachedLeftValue,cachedRightValue;
var rotator cachedCameraRot,initCameraRot;
var bool bGetInitCameraRot;
var config bool bAbsoluteCamera;

var float lastAng;
var float steerAng;
var byte count;

replication
{
	reliable if(Role == ROLE_Authority) steerAng;
}

function ProcessCarInput()
{
	local float LeftValue,RightValue;
	local int CameraPan,CameraTilt;

	Super.ProcessCarInput();

	if (!bGetInitCameraRot) {
		initCameraRot = myCamera.Rotation;
		USARRemoteBot(Controller).LeftThrottle = 0.0;
		USARRemoteBot(Controller).RightThrottle = 0.0;
		USARRemoteBot(Controller).myCameraRotation = rot(0,0,0);
		bGetInitCameraRot = true;
	}

	if (USARRemoteBot(Controller).Normalized) {
		LeftValue = USARRemoteBot(Controller).LeftThrottle*2360;
		RightValue = USARRemoteBot(Controller).RightThrottle*2360;
	} else {
		LeftValue = Converter.SpinSpeedToUU(USARRemoteBot(Controller).LeftThrottle);
		RightValue = Converter.SpinSpeedToUU(USARRemoteBot(Controller).RightThrottle);
	}

	if (cachedLeftValue!=LeftValue) {
		JointsControl[1].state = 1; // new command
		JointsControl[1].order = 1;
		JointsControl[1].value = LeftValue;
		bNewCommand = true;
		cachedLeftValue = LeftValue;
	}
	if (cachedRightValue!=RightValue) {
		JointsControl[0].state = 1; // new command
		JointsControl[0].order = 1;
		JointsControl[0].value = RightValue;
		bNewCommand = true;
		cachedRightValue = RightValue;
	}
	if (myCamera!=None) {
		if (USARRemoteBot(Controller).myCameraRotation.roll!=0) return;
		USARRemoteBot(Controller).myCameraRotation.roll = -1;
		if (USARRemoteBot(Controller).myCameraRotationOrder==1) {
			CameraTilt = USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw;
		} else if (!bAbsoluteCamera) {
			CameraTilt = -USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw;
		} else {
			CameraTilt = myCamera.Rotation.Pitch - initCameraRot.Pitch -Rotation.Pitch - USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw - myCamera.Rotation.Yaw + initCameraRot.Yaw + Rotation.Yaw;
			if (CameraTilt>65535) CameraTilt-=65535;
			else if (CameraTilt<-65535) CameraTilt+=65535;
			if (CameraTilt>32768) CameraTilt-=65535;
			else if (CameraTilt<-32768) CameraTilt+=65535;
			if (CameraPan>65535) CameraPan-=65535;
			else if (CameraPan<-65535) CameraPan+=65535;
			if (CameraPan>32768) CameraPan-=65535;
			else if (CameraPan<-32768) CameraPan+=65535;
		}
		if (CameraPan!=0 || USARRemoteBot(Controller).myCameraRotationOrder==1) {
			JointsControl[4].state = 1;
			//JointsControl[4].steer = 65535;
			JointsControl[4].order = USARRemoteBot(Controller).myCameraRotationOrder;
			JointsControl[4].value = CameraPan;
		}
		if (JointsControl[4].order==1)
			PanSpeed = CameraPan;
		else
			PanSpeed = MotorSpeed;
		if (CameraTilt!=0 || USARRemoteBot(Controller).myCameraRotationOrder==1) {
			JointsControl[5].state = 1;
			//JointsControl[5].steer = 65535;
			JointsControl[5].order = USARRemoteBot(Controller).myCameraRotationOrder;
			JointsControl[5].value = CameraTilt;
		}
		if (JointsControl[5].order==1)
			TiltSpeed = CameraTilt;
		else
			TiltSpeed = MotorSpeed;
		bNewCommand = true;
	}
	//bReady = true;
}

simulated function Tick(float Delta)
{
	local KRigidBodyState WheelState;
	local vector wheelLinVel,ChassisX;

	Super.Tick(Delta);

	if (Role == ROLE_Authority) {
		Parts[2].KGetRigidBodyState(WheelState);
		wheelLinVel = KRBVecToVector(WheelState.LinVel);
		if (VSize(wheelLinVel)<1.0) return;

		ChassisX = QuatRotateVector(KGetRBQuaternion(), vect(1, 0, 0));
		steerAng = (ACos(Normal(wheelLinVel) Dot ChassisX))*32768/3.14;
		if (abs(lastAng-steerAng)>16384) {
			if (steerAng>16384) steerAng-=32768;
			else if (steerAng<-16384) steerAng+=32768;
		}
	}

	count++;
	if (abs(lastAng-steerAng)>200 || count==4) {
		lastAng = steerAng;
		KCarWheelJoint(Joints[2]).KSteerAngle = steerAng;
	}
	if (count==4) count=0;
}

defaultproperties
{
	bDebug=false
	StaticMesh=StaticMesh'USAR_Meshes.Robot.P2DX'
	DrawScale=1.0
	DrawScale3D=(X=1.0,Y=1.0,Z=1.0)
	AmbientGlow=128

	bCollideActors=true
	bBlockActors=true
	bBlockPlayers=true
	CollisionRadius=64
	CollisionHeight=64
	
	ChassisMass=1.000000
	
	TireRollFriction=15.000000
	TireLateralFriction=15.000000
	TireRollSlip=0.0600
	TireLateralSlip=0.0600000
	TireMinSlip=0.001000
	TireSlipRate=0.00010000
	TireSoftness=0.000020
	TireAdhesion=0.000000
	TireRestitution=0.000000

    Begin Object Class=KarmaParamsRBFull Name=KParams0
        KInertiaTensor(0)=0.18000
        KInertiaTensor(3)=0.24000
        KInertiaTensor(5)=0.2600
        KCOMOffset=(X=0.0,Z=-0.3)
        KLinearDamping=0.0
        KAngularDamping=0.0
        KStartEnabled=True
        bHighDetailOnly=False
        bClientOnly=False
        bKDoubleTickRate=True
        KFriction=0.8
        Name="KParams0"
    End Object
    KParams=KarmaParamsRBFull'USARBot.P2DX.KParams0'

	JointParts(0)=(PartName="LeftFWheel",PartClass=class'USARBot.P2DXTire',bSteeringLocked=True,bSuspensionLocked=true,Parent="",JointClass=class'KCarWheelJoint',ParentPos=(X=4.2,Y=7.35,Z=-2.1),ParentAxis=(Z=1.0),SelfPos=(Z=0.0),SelfAxis=(Z=1.0))
	JointParts(1)=(PartName="RightFWheel",PartClass=class'USARBot.P2DXTire',bSteeringLocked=True,bSuspensionLocked=true,Parent="",JointClass=class'KCarWheelJoint',ParentPos=(X=4.2,Y=-7.35,Z=-2.1),ParentAxis=(Z=1.0),SelfPos=(Z=0.0),SelfAxis=(Z=1.0))
	JointParts(2)=(PartName="RearWheel",PartClass=class'USARBot.P2DXSmallTire',bSteeringLocked=True,bSuspensionLocked=true,Parent="",JointClass=class'KCarWheelJoint',ParentPos=(X=-7.35,Y=0.0,Z=-3.150),ParentAxis=(Z=1.0),SelfPos=(Z=0.0),SelfAxis=(Z=1.0))
}
