/* Sony QRIO robot
 * by Marco Zaratti - marco.zaratti@gmail.com
*/
class QRIO extends KRobot config(USAR);

var bool defaultPose;
var int loAngleLimit[21];
var int hiAngleLimit[21];
var int CameraFOV;

function ProcessCarInput()
{
	local int nJoints;

	Super.ProcessCarInput();

	//Set the starting pose
	if(!defaultPose)
	{
		//Set here default joint positions.
		//RLE
		JointsControl[13].state = 1;
		JointsControl[13].order = 0;
		JointsControl[13].value = 1000;
		//LLE
		JointsControl[19].state = 1;
		JointsControl[19].order = 0;
		JointsControl[19].value = 1000;
		//BA
		JointsControl[7].state = 1;
		JointsControl[7].order = 0;
		JointsControl[7].value = 910;
		bNewCommand = true;
		USARRemoteBot(Controller).myCameraZoom = CameraFOV;
		defaultPose=true;
	}

	//Limit joints order 0 angles
	for(nJoints=0; nJoints < JointsControl.length; nJoints++)
		if((JointsControl[nJoints].state == 1) && (JointsControl[nJoints].order == 0))
		{
			if(JointsControl[nJoints].value < loAngleLimit[nJoints])
			{
				JointsControl[nJoints].value = loAngleLimit[nJoints];
				continue;
			}
			if(JointsControl[nJoints].value > hiAngleLimit[nJoints])
				JointsControl[nJoints].value = hiAngleLimit[nJoints];
		}
}

defaultproperties
{
	bDebug=false
	CameraFOV=60

	//Low Joint Angle Limits
	loAngleLimit(0)=-20753
	loAngleLimit(1)=-31311
	loAngleLimit(2)=0
	loAngleLimit(3)=0
	loAngleLimit(4)=-31311
	loAngleLimit(5)=0
	loAngleLimit(6)=0
	loAngleLimit(7)=-10376
	loAngleLimit(8)=-5097
	loAngleLimit(9)=-14563
	loAngleLimit(10)=-9284
	loAngleLimit(11)=-5097
	loAngleLimit(12)=-2002
	loAngleLimit(13)=-7281
	loAngleLimit(14)=-7281
	loAngleLimit(15)=-14563
	loAngleLimit(16)=-9284
	loAngleLimit(17)=-5097
	loAngleLimit(18)=-2002
	loAngleLimit(19)=-7281
	loAngleLimit(20)=-7281
	//High Joint Angle Limits
	hiAngleLimit(0)=20753
	hiAngleLimit(1)=31311
	hiAngleLimit(2)=32768
	hiAngleLimit(3)=29127
	hiAngleLimit(4)=31311
	hiAngleLimit(5)=32768
	hiAngleLimit(6)=29127
	hiAngleLimit(7)=20753
	hiAngleLimit(8)=5097
	hiAngleLimit(9)=17658
	hiAngleLimit(10)=16566
	hiAngleLimit(11)=10376
	hiAngleLimit(12)=26032
	hiAngleLimit(13)=14563
	hiAngleLimit(14)=7281
	hiAngleLimit(15)=17658
	hiAngleLimit(16)=16566
	hiAngleLimit(17)=10376
	hiAngleLimit(18)=26032
	hiAngleLimit(19)=14563
	hiAngleLimit(20)=7281

	StaticMesh=StaticMesh'SampleRobotsSt.Humanoid.body'

    //Parameters of chassis. (KMass is defined in USAR.INI)
    Begin Object Class=KarmaParams Name=KParams0
        KStartEnabled=True
        bKDoubleTickRate=True
        bHighDetailOnly=False
        bClientOnly=False
        bKNonSphericalInertia=true
//Uncomment and define these when using RBfull karma params
//        KInertiaTensor(0)=0.4
//        KInertiaTensor(1)=0
//        KInertiaTensor(2)=0
//        KInertiaTensor(3)=0.4
//        KInertiaTensor(4)=0
//        KInertiaTensor(5)=0.4
//        KCOMOffset=(X=0.0,Y=0.0,Z=0.0)
        KLinearDamping=0.000000
        KAngularDamping=0.000000
        KFriction=0.8
        KRestitution=0.0
        Name="KParams0"
    End Object
    KParams=KarmaParams'KParams0'

/* USAR.INI:
[USARBot.Judo]
bHUDdebug=false
msgTimer=0.1
ChassisMass=1
MaxHingeTorque=50.0
MaxHingeSpeed=10000
HingePropGap=1820
JointParts=(PartName="HA",PartClass=class'USARBot.JudoHead',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.025,Y=0.425,Z=10.722),ParentAxis=(Z=1.0),ParentAxis2=(Y=1.0),SelfAxis=(Z=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RAA",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.026,Y=52.147,Z=25.997),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RAB",PartClass=class'USARBot.JudoRAB',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RAA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="RAC",PartClass=class'USARBot.JudoRAC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RAB",JointClass=class'USARBot.KDHinge',ParentPos=(X=5.911,Y=-0.002,Z=-47.306),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LAA",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.026,Y=-51.305,Z=25.999),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LAB",PartClass=class'USARBot.JudoLAB',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LAA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="LAC",PartClass=class'USARBot.JudoLAC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LAB",JointClass=class'USARBot.KDHinge',ParentPos=(X=5.911,Y=-0.005,Z=-47.304),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="BA",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.026,Y=0.425,Z=-44.45),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="BB",PartClass=class'USARBot.JudoBack',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="RLA",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=23.64,Z=-27.587),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="RLB",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RLC",PartClass=class'USARBot.JudoLEC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="RLD",PartClass=class'USARBot.JudoLED',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLC",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=-57.157),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RLE",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLD",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.943,Y=0.0,Z=-51.217),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RLF",PartClass=class'USARBot.JudoLEF',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLE",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LLA",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=-23.64,Z=-27.587),ParentAxis=(Z=1.0),ParentAxis2=(Y=1.0),SelfAxis=(Z=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="LLB",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LLC",PartClass=class'USARBot.JudoLEC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="LLD",PartClass=class'USARBot.JudoLED',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLC",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=-57.157),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LLE",PartClass=class'USARBot.JudoDummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLD",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.943,Y=0.0,Z=-51.217),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LLF",PartClass=class'USARBot.JudoLEF',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLE",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
*/
}
