// ===============================================================
// USARBot.RFIDTag
//
// This class simulates the RFID tag. The variable 'id' is the tag's
// id number. The tag automatically registers to the game to help 
// the RFID reader quickly search all the tags and decide which tag
// should be detected.
//
// Original created by: Alexander Kleiner and Christian 
// Modified by: Jijun Wang   -- 12/15/2005
// ===============================================================

class RFIDTag extends Actor placeable;

var () int id;
var () bool bSingleShot;

// Register to the game
function Register()
{
	local USARDeathMatch UsarGame;
	local int Index;

	UsarGame = USARDeathMatch(Level.Game);
	Index = UsarGame.RFIDTags.length;
	UsarGame.RFIDTags.Insert(Index, 1);
	UsarGame.RFIDTags[Index] = self;
	if(UsarGame.bAutoId)
		UsarGame.RFIDTags[Index].id = Index;
}

// Unregister from the game
function Unregister()
{
	local USARDeathMatch UsarGame;
	local int i;

	UsarGame = USARDeathMatch(Level.Game);
	for (i = 0; i < UsarGame.RFIDTags.length; i++) {
		if (UsarGame.RFIDTags[i] == self) {
			UsarGame.RFIDTags.Remove(i, 1);
			break;
		}
	}
}

simulated function PostNetBeginPlay()
{
	super.PostNetBeginPlay();
	Register();
}

simulated event Destroyed()
{
	Unregister();
	Super.Destroyed();
}

defaultproperties
{
	bNoDelete=False
	bCollideWorld=True
	CollisionRadius=0.00001
	CollisionHeight=0.00001
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'Editor.TexPropCube'
	DrawScale3D=(X=0.01,Y=0.01,Z=0.002)
	bStatic=False
	bStasis=False
	bSingleShot=True
	Physics=PHYS_Falling
}
