// ===============================================================
// USARBot.RFIDVictSensor
//
// Added additional functionality for the Automatic Referee
// Mentar Mahmudi - International University Bremen - m.mahmudi@iu-bremen.de
// ===============================================================

class RFIDVictSensor extends sensor config(USAR);

//distance for the ranges
var config float VictMinRange;
var config float FPMinRange;
var config float MaxRange;
var float uuVictMinRangeSquare;
var float uuFPMinRangeSquare;
var float uuMaxRangeSquare;
var config float RFIDFOV;  // default value is around 30 deg.

function ConvertParam(USARConverter converter)
{
    if (converter != None) {
        uuMaxRangeSquare = converter.LengthToUU(MaxRange) * converter.LengthToUU(MaxRange);
		uuVictMinRangeSquare = converter.LengthToUU(VictMinRange) * converter.LengthToUU(VictMinRange);
		uuFPMinRangeSquare = converter.LengthToUU(FPMinRange) * converter.LengthToUU(FPMinRange); 
    } else {
        uuMaxRangeSquare = MaxRange*MaxRange;
		uuVictMinRangeSquare = VictMinRange*VictMinRange;
		uuFPMinRangeSquare = FPMinRange*FPMinRange;
    }
}

function float VectLenSq(vector v) {
	return v.x * v.x + v.y * v.y + v.z * v.z;
}

function String GetData()
{
	local string Outstring;
	local int i;
	local vector diff, pos;
	local array<VictRFIDTag> tags;

	Outstring="";
	
	tags = USARDeathMatch(Level.Game).VictRFIDTags;
	
	//Prints the position of the tag(s) in terms of the coordinate system with
	//the robot tag sensor being at the origin.
	
	for (i = 0; i < tags.length; i++) {
		diff = tags[i].Location - Location;
		if (VectLenSq(diff) < uuMaxRangeSquare) {
			pos = tag_pos(tags[i]);
			if(inFOV(pos)) { //checks whether the tag is in the FOV of the robot
				if(tags[i].id == -1) {
					//false positive
					if(VectLenSq(diff) < uuFPMinRangeSquare)
						Outstring = Outstring$" {ID FalsePositive} {Location "$converter.VectorString(pos, 2)$"}";
					else
						Outstring = Outstring$" {ID None} {Location "$converter.VectorString(pos, 2)$"}";
				}
				else {
					//victim positive
					if(VectLenSq(diff) < uuVictMinRangeSquare)
						Outstring = Outstring$" {ID "$tags[i].name$"} {Location "$converter.VectorString(pos, 2)$"}";
					else
						Outstring = Outstring$" {ID None} {Location "$converter.VectorString(pos, 2)$"}";
				}
			}
		}
	}
		
	if (Outstring != "") Outstring = "{Name "$ItemName$"}"$OutString;
	if (Outstring != "") {
		//log(Outstring);
		return Outstring;
	}
}

function vector tag_pos(VictRFIDTag aTag) {

	local vector tag_pos, robot_pos, VX, VY, VZ, tmp, pos;
	local Rotator robot_rot;
	local Matrix T;
	local Plane PX, PY, PZ, PW;
	
		tag_pos = aTag.Location;
		robot_pos = Location;
		robot_rot = Rotation;
			
			// calculating the position of the tag
			GetUnAxes (robot_rot, VX, VY, VZ);
			PX.x = VX.x; PX.y = VX.y; PX.z = VX.z; PX.W = 0;
			PY.x = VY.x; PY.y = VY.y; PY.z = VY.z; PY.W = 0;
			PZ.x = VZ.x; PZ.y = VZ.y; PZ.z = VZ.z; PZ.W = 0;
			tmp.x = VX.x;
			tmp.y = VY.x;
			tmp.z = VZ.x;
			PW.x = - ( tmp Dot robot_pos ); 
			tmp.x = VX.y;
			tmp.y = VY.y;
			tmp.z = VZ.y;
			PW.y = - ( tmp Dot robot_pos);
			tmp.x = VX.z;
			tmp.y = VY.z;
			tmp.z = VZ.z;
			PW.z = - ( tmp Dot robot_pos); 
			PW.W = 1;
			T.XPlane = PX;
			T.YPlane = PY;
			T.ZPlane = PZ;
			T.WPlane = PW;
			
	//use converter here
	pos.x = converter.LengthFromUU(T.XPlane.X * tag_pos.x + T.YPlane.x * tag_pos.y + T.zplane.x * tag_pos.z + T.wplane.x);
	pos.y = converter.LengthFromUU(T.XPlane.Y * tag_pos.x + T.YPlane.Y * tag_pos.y + T.zplane.Y * tag_pos.z + T.wplane.Y);
	pos.z = converter.LengthFromUU(-(T.XPlane.Z * tag_pos.x + T.YPlane.Z * tag_pos.y + T.zplane.Z * tag_pos.z + T.wplane.Z));
			
	return pos;
			
	//yaw = robot_rot.yaw*2*Pi/65536;
			//the calculation is correct.
			//x = ((tag_pos.x - robot_pos.x) * cos(yaw) + (tag_pos.y - robot_pos.y) * sin(yaw))/52.5;
			//y = (-(tag_pos.x - robot_pos.x) * sin(yaw) + (tag_pos.y - robot_pos.y) * cos(yaw))/52.5;
}

function String GetConfData()
{
	local string outstring;
	outstring = Super.GetConfData();
	outstring = outstring@"{MaxRange "$MaxRange$"} {VictMinRange "$VictMinRange$"} {FPMinRange "$FPMinRange$"}";
	return outstring;
}

function bool inFOV(vector pos) {
	local float angle;

	angle = ATan(pos.x, pos.y);
	
	if((angle < (pi/2 - RFIDFOV/2)) || (angle > (pi/2 + RFIDFOV/2)))
		return false;
	
	angle = ATan(pos.x, pos.z);
	
	if((angle < (pi/2 - RFIDFOV/2)) || (angle > (pi/2 + RFIDFOV/2)))
		return false;
		
	return true;
}

defaultproperties
{
     	MaxRange=1.6
     	ItemType="VictRFID"
	VictMinRange=0.5
	FPMinRange=1
	RFIDFOV=0.52
}
