/*
    This script allows to move the ball around the map. SetLocation function
    is not enough becouse it updates only the mesh position, note the rigid
    body state. That's why we need to update manually rigid body state. Here's
    another problem: if we move the ball near the previous position (100~200uu)
    than movement will not be precise. It seems that Karma applies an impulse to
    the ball. Only if we move the ball far away the position is correctly updated.
    I don't know the reason of this behavior, for now I'll hack movement code
    making a 2 step movement: 1) I'll move the ball far away both from current and
    destination position. 2) I'll move the ball to destination position.

     by Marco Zaratti - marco.zaratti@gmail.com
*/
class SoccerBall extends KActor config(USAR);

var KRigidBodyState BallRB;
var int BallState; //0: normal, 1: ready to move, 2: in "space"
var vector newPos, tempPos;

simulated event bool KUpdateState(out KRigidBodyState newState)
{
    if(BallState == 0)
		return false;

	newState = BallRB;

		if(BallState == 2)
		   BallState = 0;
		else
		{
            BallRB.Position = KRBVecFromVector(newPos);
            BallRB.AngVel = KRBVecFromVector(vect(0,0,0));
            BallRB.LinVel = KRBVecFromVector(vect(0,0,0));
            BallState++;
        }

	return true;
}

simulated function MoveBall(float posX, float posY, float posZ)
{
    newPos.X = posX; newPos.Y = posY; newPos.Z = posZ;

    KGetRigidBodyState(BallRB);
    BallRB.Position = KRBVecFromVector(tempPos);
    BallRB.AngVel = KRBVecFromVector(vect(0,0,0));
    BallRB.LinVel = KRBVecFromVector(vect(0,0,0));
    BallState++;
}

defaultproperties
{
    tempPos=(X=0,Y=0,Z=900) //Temporary position for ball movement
	StaticMesh=StaticMesh'RobotSoccerStatic.SoccerField.Ball_8cm'
    Begin Object Class=KarmaParams Name=KBall
        KStartEnabled=True
        bKDoubleTickRate=True
        bHighDetailOnly=False
        bClientOnly=False
        KMass=0.08
        KLinearDamping=0.15
        KAngularDamping=0.1
        KMaxAngularSpeed=655360
        KMaxSpeed=25000
        KFriction=0.05
        KRestitution=0.9
        Name="KBall"
    End Object
    KParams=KarmaParams'USARBot.SoccerBall.KBall'
}
