class USARVictim extends xIntroPawn;

struct Segment {
	var() name SegName;
	var() float Scale;
	var() rotator InitRotation;
	var() rotator FinalRotation;
	var() int PitchRate;
	var() int YawRate;
	var() int RollRate;
};

struct SegmentRot {
	var rotator curRot;
	var vector increase;
};

var(Victim) Sound HelpSound;
var(Victim) Segment Segments[8];
var(Victim) float AnimTimer;

var float SoundT,SoundDT,AnimT;
var bool bInit;
var SegmentRot SegRot[8];
var vector SegPos[8];
var vector SegVel[8];

replication
{
	reliable if(Role == ROLE_Authority)
		HelpSound, Segments, AnimTimer;
}

// Register to the game
function Register()
{
	local USARDeathMatch UsarGame;
	local int Index;

	UsarGame = USARDeathMatch(Level.Game);
	Index = UsarGame.Victims.length;
	UsarGame.Victims.Insert(Index,1);
	UsarGame.Victims[Index]=self;
}

// Unregister from the game
function Unregister()
{
	local USARDeathMatch UsarGame;
	local int i;
	
	UsarGame = USARDeathMatch(Level.Game);
	for (i=0;i<UsarGame.Victims.length;i++) {
		if (UsarGame.Victims[i]==self) {
			UsarGame.Victims.Remove(i,1);
			break;
		}
	}
}

function vector getSegmentVel(int idx)
{
	return SegVel[idx];
}

function vector getSegmentLocation(int idx)
{
	return GetBoneCoords(Segments[idx].SegName).Origin;
}

simulated function PostNetBeginPlay()
{
	
	SoundDT=GetSoundDuration(HelpSound)+1.0;
	SoundT=Level.TimeSeconds;
	AnimT=Level.TimeSeconds;
	Register();
}

simulated event Destroyed()
{
	Unregister();
}

simulated function Tick(float delta) {
	local int i;
	local float dif;
	
	if (!bInit) initVictim();

	if (Role != ROLE_Authority && Level.TimeSeconds-SoundT>SoundDT) {
		PlaySound(HelpSound,SLOT_None,SoundVolume,true,SoundRadius,SoundPitch,false);
		SoundT=Level.TimeSeconds;
	}

	if (Level.TimeSeconds-AnimT<AnimTimer) return;
	AnimT=Level.TimeSeconds;
	for (i=0;i<8;i++) {
		if (Segments[i].PitchRate!=0||Segments[i].YawRate!=0||Segments[i].RollRate!=0) {
			SegRot[i].curRot.Pitch+=Segments[i].PitchRate*SegRot[i].increase.X;
			if (SegRot[i].increase.X>0) {
				dif=abs(SegRot[i].curRot.Pitch-Segments[i].FinalRotation.Pitch);
				if (dif>32768.0) dif=65535.0-dif;
				if (dif<abs(Segments[i].PitchRate))
					SegRot[i].increase.X=-1;
			}
			else {
				if (abs(SegRot[i].curRot.Pitch-Segments[i].InitRotation.Pitch)<abs(Segments[i].PitchRate))
					SegRot[i].increase.X=1;
			}

			SegRot[i].curRot.Yaw+=Segments[i].YawRate*SegRot[i].increase.Y;
			if (SegRot[i].increase.Y>0) {
				dif=abs(SegRot[i].curRot.Yaw-Segments[i].FinalRotation.Yaw);
				if (dif>32768) dif=65535-dif;
				if (dif<abs(Segments[i].YawRate))
					SegRot[i].increase.Y=-1;
			}
			else {
				if (abs(SegRot[i].curRot.Yaw-Segments[i].InitRotation.Yaw)<abs(Segments[i].YawRate))
					SegRot[i].increase.Y=1;
			}

			SegRot[i].curRot.Roll+=Segments[i].RollRate*SegRot[i].increase.Z;
			if (SegRot[i].increase.Z>0) {
				dif=abs(SegRot[i].curRot.Roll-Segments[i].FinalRotation.Roll);
				if (dif>32768) dif=65535-dif;
				if (dif<abs(Segments[i].RollRate))
					SegRot[i].increase.Z=-1;
			}
			else {
				if (abs(SegRot[i].curRot.Roll-Segments[i].InitRotation.Roll)<abs(Segments[i].RollRate))
					SegRot[i].increase.Z=1;
			}

			SetBoneRotation(Segments[i].SegName,SegRot[i].curRot,1.0);
			SegVel[i] = (SegPos[i] - GetBoneCoords(Segments[i].SegName).Origin)/delta;
			SegPos[i] = GetBoneCoords(Segments[i].SegName).Origin;
			/*
			log("i="$i$" Name="$Segments[i].SegName$
				" InitRotation="$Segments[i].InitRotation$
				" FinalRotation="$Segments[i].FinalRotation$
				" PitchRate="$Segments[i].PitchRate$
				" YawRate="$Segments[i].YawRate$
				" RollRate="$Segments[i].RollRate$
				" SegRot_increase="$SegRot[i].increase$
				" SegRot_curRot="$SegRot[i].curRot);
			*/
		}
	}
}

simulated function initVictim() {
	local int i;

	for (i=0;i<8;i++) {
		//log("Segment"@Segments[i].SegName);
		if (Segments[i].SegName!='') {
			SetBoneScale(i,Segments[i].Scale,Segments[i].SegName);
			if (Segments[i].InitRotation.Pitch!=0||Segments[i].InitRotation.Yaw!=0||Segments[i].InitRotation.Roll!=0)
				SetBoneRotation(Segments[i].SegName,Segments[i].InitRotation,1.0);
			if (Segments[i].PitchRate!=0||Segments[i].YawRate!=0||Segments[i].RollRate!=0) {
				SegRot[i].curRot=Segments[i].InitRotation;
				SegRot[i].increase=vect(1,1,1);
			}
		}
	}
	bInit=true;
}
function Touch( actor Other )
{
   Log("Victim: I was touched!");
}
defaultproperties
{
    bClientAnim=false
    bNoRepMesh=false
    DrawScale=1.500000
    SoundRadius=5.000000
    SoundVolume=24
    SoundPitch=1
    Mesh=SkeletalMesh'Intro_gorgefan.Intro_gorgefan'
	bBlockPlayers=true
    bCollideActors=True
	CollisionRadius=64
	CollisionHeight=64
}
